/* --COPYRIGHT--,BSD
 * Copyright (c) 2015, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/
/*
 * ======== hal.c ========
 *
 */

#include "hal.h"	// All other required source files are declared here

#define GPIO_ALL	GPIO_PIN0|GPIO_PIN1|GPIO_PIN2|GPIO_PIN3| \
					GPIO_PIN4|GPIO_PIN5|GPIO_PIN6|GPIO_PIN7



/*
* This function drives all the I/O's as output-low, to avoid floating inputs
* (which cause extra power to be consumed).  This setting is compatible with  
 * TI FET target boards, the F5529 Launchpad, and F5529 Experimenters Board;  
 * but may not be compatible with custom hardware, which may have components  
 * attached to the I/Os that could be affected by these settings.  So if using
* other boards, this function may need to be modified.
*/

//bufferType_t buffer_q[8];
volatile uint8_t g_DRDY_flag = 0;			//declared in hal.h, defined here
//unsigned char g_outputFormat = FORMAT_INT;		//declared in hal.h, defined here
//uint16_t g_excitationCounter;
//uint8_t g_packetCounter = 0;


void initPorts(void)
{
#ifdef __MSP430_HAS_PORT1_R__
    GPIO_setOutputLowOnPin(GPIO_PORT_P1, GPIO_ALL);
    GPIO_setAsOutputPin(GPIO_PORT_P1, GPIO_ALL);
#endif

#ifdef __MSP430_HAS_PORT2_R__
    GPIO_setOutputLowOnPin(GPIO_PORT_P2, GPIO_ALL);
    GPIO_setAsOutputPin(GPIO_PORT_P2, GPIO_ALL);
#endif

#ifdef __MSP430_HAS_PORT3_R__
    GPIO_setOutputLowOnPin(GPIO_PORT_P3, GPIO_ALL);
    GPIO_setAsOutputPin(GPIO_PORT_P3, GPIO_ALL);
#endif

#ifdef __MSP430_HAS_PORT4_R__
    GPIO_setOutputLowOnPin(GPIO_PORT_P4, GPIO_ALL);
    GPIO_setAsOutputPin(GPIO_PORT_P4, GPIO_ALL);
#endif

#ifdef __MSP430_HAS_PORT5_R__
    GPIO_setOutputLowOnPin(GPIO_PORT_P5, GPIO_ALL);
    GPIO_setAsOutputPin(GPIO_PORT_P5, GPIO_ALL);
#endif

#ifdef __MSP430_HAS_PORT6_R__
    GPIO_setOutputLowOnPin(GPIO_PORT_P6, GPIO_ALL);
    GPIO_setAsOutputPin(GPIO_PORT_P6, GPIO_ALL);
#endif

#ifdef __MSP430_HAS_PORT7_R__
    GPIO_setOutputLowOnPin(GPIO_PORT_P7, GPIO_ALL);
    GPIO_setAsOutputPin(GPIO_PORT_P7, GPIO_ALL);
#endif

#ifdef __MSP430_HAS_PORT8_R__
    GPIO_setOutputLowOnPin(GPIO_PORT_P8, GPIO_ALL);
    GPIO_setAsOutputPin(GPIO_PORT_P8, GPIO_ALL);
#endif

#ifdef __MSP430_HAS_PORT9_R__
    GPIO_setOutputLowOnPin(GPIO_PORT_P9, GPIO_ALL);
    GPIO_setAsOutputPin(GPIO_PORT_P9, GPIO_ALL);
#endif

#ifdef __MSP430_HAS_PORTJ_R__
    GPIO_setOutputLowOnPin(GPIO_PORT_PJ, GPIO_ALL);
    GPIO_setAsOutputPin(GPIO_PORT_PJ, GPIO_ALL);
#endif

// Set SPI pins as peripheral module
GPIO_setAsPeripheralModuleFunctionOutputPin(SPI_PORT, MOSI_PIN | SCLK_PIN);
GPIO_setAsPeripheralModuleFunctionInputPin(SPI_PORT, MISO_PIN);

// CS pin
GPIO_setOutputHighOnPin(CS_PORT, CS_PIN); 				//Start state of CS is high
GPIO_setAsOutputPin(CS_PORT, CS_PIN);					//Set CS as output

// RST_PWDN pin
GPIO_setOutputHighOnPin(RST_PWDN_PORT, RST_PWDN_PIN); 	// Start state of PWDN is high
GPIO_setAsOutputPin(RST_PWDN_PORT, RST_PWDN_PIN);		// Set PWDN as output

// START pin
GPIO_setOutputHighOnPin(START_PORT, GPIO_PIN7);			// Start state of START is high
GPIO_setAsOutputPin(START_PORT, GPIO_PIN7);				// Set START as output

// DRDY pin
GPIO_setAsInputPin(DRDY_PORT, DRDY_PIN);				// Set DRDY pin as input
//GPIO_enableInterrupt(DRDY_PORT, DRDY_PIN);			// Enable interrupt on DRDY
GPIO_interruptEdgeSelect(DRDY_PORT, DRDY_PIN, GPIO_HIGH_TO_LOW_TRANSITION);	//interrupt on high-to-low edge

//GPIO_setAsOutputPin(GPIO_PORT_P3, GPIO_PIN3 | GPIO_PIN4);	// Set UART pins as output
//GPIO_setAsOutputPin(GPIO_PORT_P6, GPIO_PIN5 | GPIO_PIN6);
//GPIO_setOutputLowOnPin(GPIO_PORT_P3, GPIO_PIN3 | GPIO_PIN4);
//GPIO_setOutputLowOnPin(GPIO_PORT_P6, GPIO_PIN5 | GPIO_PIN6);
}

/* Configures the system clocks:
* MCLK = SMCLK = DCO/FLL = mclkFreq (expected to be expressed in Hz)
* ACLK = FLLref = REFO=32kHz
*
* XT2 is not configured here.  Instead, the USB API automatically starts XT2
* when beginning USB communication, and optionally disables it during USB
* suspend.  It's left running after the USB host is disconnected, at which
* point you're free to disable it.  You need to configure the XT2 frequency
* in the Descriptor Tool (currently set to 4MHz in this example).
* See the Programmer's Guide for more information.
*/
void initClocks(void)
{
	PMM_setVCore(PMM_CORE_LEVEL_3);								// Increase core voltage to support 25MHz
	GPIO_setAsPeripheralModuleFunctionInputPin(GPIO_PORT_P5, 	// Set crystal pins as peripheral module
											   GPIO_PIN2 |
											   GPIO_PIN3 |
											   GPIO_PIN4 |
											   GPIO_PIN5);

	UCS_setExternalClockSource(XT1FREQ, XT2FREQ);							// Let UCS know about our crystal frequencies
	UCS_LFXT1Start(UCS_XT1_DRIVE3, UCS_XCAP_0);								// Start XT1 (no added capacitance)
	UCS_XT2Start(UCS_XT2DRIVE_4MHZ_8MHZ);									// Start XT2
	UCS_clockSignalInit(UCS_FLLREF,UCS_XT1CLK_SELECT,UCS_CLOCK_DIVIDER_1);	// Define XT1 as source for FLL
	UCS_clockSignalInit(UCS_ACLK,UCS_XT1CLK_SELECT,UCS_CLOCK_DIVIDER_1);	// Define XT1 as source for ACLK
	UCS_initFLLSettle(SYSFREQ/1000,SYSFREQ/XT1FREQ);						// Start FLL with the defined SYSFREQ
	UCS_clockSignalInit(UCS_SMCLK,UCS_DCOCLK_SELECT,UCS_CLOCK_DIVIDER_1);	// Define DCO/FLL as source for SMCLK
}


//Configures MSP430 SPI peripheral
void initSPI(void)
{
	USCI_B_SPI_masterInit(USCI_B0_BASE,								// Initialize SPI. Clock 3MHz, based on SMCLK
						  USCI_B_SPI_CLOCKSOURCE_SMCLK,
						  UCS_getSMCLK(),
						  3000000,
						  USCI_B_SPI_MSB_FIRST,
						  USCI_B_SPI_PHASE_DATA_CHANGED_ONFIRST_CAPTURED_ON_NEXT,
						  USCI_B_SPI_CLOCKPOLARITY_INACTIVITY_LOW);
	USCI_B_SPI_enable(USCI_B0_BASE);								// Enable SPI
	USCI_B_SPI_disableInterrupt(USCI_B0_BASE,						// Disable all SPI interrupts
								USCI_B_SPI_TRANSMIT_INTERRUPT |
								USCI_B_SPI_RECEIVE_INTERRUPT);
}


/*
 * ======== initTimer ========
 */
void initTimer (void)
{
    // Start timer
    TIMER_A_clearTimerInterruptFlag(TIMER_A0_BASE);

    TIMER_A_configureUpMode(TIMER_A0_BASE,
        TIMER_A_CLOCKSOURCE_ACLK,
        TIMER_A_CLOCKSOURCE_DIVIDER_1,
		SLOW_TOOGLE_PERIOD, //0,
        TIMER_A_TAIE_INTERRUPT_DISABLE,
        TIMER_A_CAPTURECOMPARE_INTERRUPT_ENABLE,
        TIMER_A_DO_CLEAR
        );
}


//Enable /DRDY interrupt, wait for /DRDY's, then clear interrupt
void WaitForDRDY(void)
{
	g_DRDY_flag = 0;
	GPIO_clearInterruptFlag(DRDY_PORT, DRDY_PIN);
	GPIO_enableInterrupt(DRDY_PORT, DRDY_PIN);

	while(!g_DRDY_flag);

	GPIO_disableInterrupt(GPIO_PORT_P1, GPIO_PIN2);
	g_DRDY_flag = 0;
}


